/*
 * $Id: driver.c,v 1.1 2001/11/07 23:05:39 joe Exp $
 *
 * Example driver program for running FLIGHTLAB standalone models.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "example.h"
#include "fcm.h"

static FILE *open_input(const char *filename);
static FILE *open_output(const char *filename);
static int read_input(FILE *fpin, struct CONTROLS *controls);
static void write_output(FILE *fpout, struct STATES *states);

/*
 * Main routine:
 */
int main(int argc,char *argv[])
{
    const char *fcm_filename	= "example.fcm";
    const char *input_filename	= "indata.tab";
    const char *output_filename	= "outdata.tab";
    FILE *fpin, *fpout;

    struct STATES	states;
    struct CONTROLS	controls;

    FCM_FILE		fcm;
    FCM_MODEL		model;
    FCM_VARLIST		vl_controls, vl_states;
    FCM_OPERATION	op_step;

    controls.xa = controls.xb = controls.xc = controls.xp = 0.0;

    /*
     * Initialize model resources:
     */
    fprintf(stderr, "Initializing...\n");
    if (!(  (fcm 	= fcm_open(fcm_filename))
	 && (vl_controls= fcm_lookup_varlist(fcm, "@CONTROLS"))
	 && (vl_states	= fcm_lookup_varlist(fcm, "@STATES"))
	 && (op_step	= fcm_lookup_operation(fcm, "STEP"))
	 && (model	= fcm_create(fcm))
    ))
    {
    	fcm_perror("Initialization failure");
	return EXIT_FAILURE;
    }

    /*
     * Open data files:
     */
    fpin = open_input(input_filename);
    if (!fpin)
	return EXIT_FAILURE;

    fpout = open_output(output_filename);
    if (!fpout)
    	return EXIT_FAILURE;

    /*
     * Run the model:
     */
    fprintf(stderr, "Running...\n");
    while (read_input(fpin, &controls) != 0)
    {
    	fcm_write(model, vl_controls, &controls, sizeof(controls));
	fcm_invoke(model, op_step);
	fcm_read(model, vl_states, &states, sizeof(states));
	write_output(fpout, &states);
    }

    /*
     * Clean up.
     */
    fprintf(stderr, "Done.\n");
    fcm_close(fcm);
    fclose(fpin);
    fclose(fpout);

    return EXIT_SUCCESS;
}

/*
 * open_input --
 *	Opens the input file for reading and scans past header information.
 */
static FILE *open_input(const char *filename)
{
    char buf[255];
    FILE *fpin = fopen(filename, "r");

    if (!fpin) {
    	perror(filename);
	return 0;
    }

    while (fgets(buf, 255, fpin) != NULL) {
	if (strncmp(buf, "!T ", 3) == 0) {
	    /* Found the beginning of the data */
	    break;
	}
    }

    return fpin;
}

/*
 * read_input --
 *	Reads one input record.
 * 	Returns: 1 if OK, 0 if end-of-file.
 */
int read_input(FILE *fpin, struct CONTROLS *controls)
{
    return (fscanf(fpin, "%lf\n", &controls->xb) == 1);
}

/*
 * open_output --
 *	Opens output file and writes initial header.
 */

static FILE *open_output(const char *filename)
{
    FILE *fpout = fopen(filename, "w");
    if (!fpout) {
    	perror(filename);
	return 0;
    }

    fprintf(fpout,"#.TAB\n");
    fprintf(fpout,"!T PHI THETA PSI P Q R\n");

    return fpout;
}

/* write_output --
 *	Writes one output record.
 */
static void write_output(FILE *fpout, struct STATES *states)
{
    fprintf(fpout, " %f %f %f  %f %f %f\n",
	states->phi, states->theta, states->psi,
	states->p, states->q, states->r);
}

/*EOF */
